package com.ngrob.android.bluemoon.core.data.repository

import android.util.Log
import com.ngrob.android.bluemoon.core.database.dao.BleedingDao
import com.ngrob.android.bluemoon.core.database.model.Bleeding
import com.ngrob.android.bluemoon.core.database.model.BleedingStrength
import com.ngrob.android.bluemoon.core.reminder.BluemoonReminderScheduler
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.flow.firstOrNull
import kotlinx.coroutines.flow.map
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.YearMonth
import java.time.format.DateTimeFormatter
import javax.inject.Inject


class OfflineBleedingRepository @Inject constructor(
    private val bleedingDao: BleedingDao,
    private val userDataRepository: UserDataRepository,
    private val bluemoonReminderScheduler: BluemoonReminderScheduler
) : BleedingRepository {

    override fun getAllBleedingsOfMonth(month: YearMonth): Flow<List<LocalDate>> {
        val startDate = month.atDay(1)
        val endDate = month.atEndOfMonth()
        val bleedingFlow = bleedingDao.getAllBleedingDaysOfMonth(
            startDate.format(DateTimeFormatter.ISO_LOCAL_DATE),
            endDate.format(DateTimeFormatter.ISO_LOCAL_DATE)
        )
        return bleedingFlow.map { dateStrings ->
            val bleedingDays: MutableList<LocalDate> = mutableListOf();
            dateStrings.forEach { value ->
                bleedingDays.add(LocalDate.parse(value, DateTimeFormatter.ISO_LOCAL_DATE))
            }
            bleedingDays.toList()
        }
    }

    override fun getAllBleedingDates(): Flow<List<LocalDate>> {
        val bleedingFlow = bleedingDao.getAllBleedingDates()
        return bleedingFlow.map { dateStrings ->
            val bleedingDays: MutableList<LocalDate> = mutableListOf();
            dateStrings.forEach { value ->
                bleedingDays.add(LocalDate.parse(value, DateTimeFormatter.ISO_LOCAL_DATE))
            }
            bleedingDays.toList()
        }
    }

    override fun getBleedingEntryOfDay(day: LocalDate): Flow<Bleeding?> {
        return bleedingDao.getBleedingOfDay(day.format(DateTimeFormatter.ISO_LOCAL_DATE))
    }

    override suspend fun addBleeding(strength: BleedingStrength, date: LocalDate) {
        val bleedingEntity = Bleeding(
            id = null,
            strength = strength,
            date = date.format(DateTimeFormatter.ISO_LOCAL_DATE),
            loggedAt = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
        )
        bleedingDao.insertBleedings(bleedingEntity)
        rescheduleUpcomingPeriodReminder()
    }

    override suspend fun updateBleeding(strength: BleedingStrength, date: LocalDate) {
        var bleedingEntity =
            bleedingDao.getBleedingOfDay(date.format(DateTimeFormatter.ISO_LOCAL_DATE))
                .firstOrNull()

        Log.d("OfflineBleedingRepository", "bleedingEntity: $bleedingEntity")
        if (bleedingEntity == null) {
            bleedingDao.upsertBleedings(
                Bleeding(
                    id = null,
                    strength = strength,
                    date = date.format(DateTimeFormatter.ISO_LOCAL_DATE),
                    loggedAt = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
                )
            )
            Log.d("OfflineBleedingRepository", "new bleeing added")
            return
        }
        Log.d("OfflineBleedingRepository", "updated Bleeding")
        bleedingEntity.strength = strength
        bleedingEntity.loggedAt = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
        bleedingDao.upsertBleedings(bleedingEntity)
        rescheduleUpcomingPeriodReminder()
    }

    override suspend fun addBleedings(dates: List<LocalDate>) {
        val entities = dates.map { date ->
            val bleedingEntity = Bleeding(
                id = null,
                strength = null,
                date = date.format(DateTimeFormatter.ISO_LOCAL_DATE),
                loggedAt = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
            )
            bleedingEntity
        }
        bleedingDao.insertBleedings(*entities.toTypedArray())
        rescheduleUpcomingPeriodReminder()
    }

    override suspend fun deleteBleedings(dates: List<LocalDate>) {
        val datesToDeleteIso = dates.map { date ->
            date.format(DateTimeFormatter.ISO_LOCAL_DATE)
        }
        bleedingDao.deleteBleedingDates(datesToDeleteIso)
        rescheduleUpcomingPeriodReminder()
    }

    override suspend fun deleteBleedingById(id: Int) {
        bleedingDao.deleteBleedingById(id)
        rescheduleUpcomingPeriodReminder()
    }

    override suspend fun deleteAllBleeding() {
        bleedingDao.deleteAllBleeding()
        rescheduleUpcomingPeriodReminder()
    }

    private suspend fun rescheduleUpcomingPeriodReminder() {
        userDataRepository.userData.collectLatest { userData ->
            getAllBleedingDates().collectLatest { bleedings ->
                bluemoonReminderScheduler.scheduleUpcomingPeriodReminder(userData, bleedings)
            }
        }
    }
}