package com.ngrob.android.bluemoon.features.calendar.screens

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.ngrob.android.bluemoon.MenstruationCycleTracker
import com.ngrob.android.bluemoon.core.data.repository.BleedingRepository
import com.ngrob.android.bluemoon.core.data.repository.PillRepository
import com.ngrob.android.bluemoon.core.data.repository.SexRepository
import com.ngrob.android.bluemoon.core.domain.GetAllBleedingDaysOfMonthUseCase
import com.ngrob.android.bluemoon.features.calendar.components.CardEntryItem
import com.ngrob.android.bluemoon.features.calendar.components.TrackingTypes
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.Job
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.emptyFlow
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.YearMonth
import java.time.format.DateTimeFormatter
import javax.inject.Inject

data class TrackedItem(
    val id: Int,
    val cardEntryItem: CardEntryItem,
    val values: Map<String, List<String>>,
    val loggedAt: LocalDateTime
)

data class CalendarUiState(
    val selectedDay: LocalDate = LocalDate.now(),
    val currentMonth: YearMonth = YearMonth.now(),
    val bleedingDays: Flow<List<LocalDate>> = emptyFlow(),
    val dayEntries: Flow<List<TrackedItem>> = emptyFlow(),
    val bleedingDaysPredicted: List<LocalDate> = emptyList(),
    val fabVisible: Boolean = true
)

@HiltViewModel
class CalendarViewModel @Inject constructor(
    private val getAllBleedingDaysOfMonth: GetAllBleedingDaysOfMonthUseCase,
    private val sexRepository: SexRepository,
    private val bleedingRepository: BleedingRepository,
    private val pillRepository: PillRepository
) : ViewModel() {
    private val _uiState = MutableStateFlow(CalendarUiState())
    val uiState = _uiState.asStateFlow()
    private var fetchBleeding: Job? = null
    private var fetchEntries: Job? = null
    private var menstruationCycleTracker: MenstruationCycleTracker = MenstruationCycleTracker()

    init {
        selectDay(LocalDate.now())
        viewModelScope.launch {
            val bleedingDays = withContext(Dispatchers.IO) {
                bleedingRepository.getAllBleedingDates()
            }
            bleedingDays.collect { bleedingDates ->
                menstruationCycleTracker = MenstruationCycleTracker(bleedingDates)
                val newPredictedBleeding =
                    menstruationCycleTracker.predictPeriodDaysOfMonth(YearMonth.now())
                _uiState.update { currentState ->
                    currentState.copy(bleedingDaysPredicted = newPredictedBleeding)
                }

            }
        }
    }

    fun selectDay(newDay: LocalDate) {
        fetchEntries?.cancel()
        fetchEntries = viewModelScope.launch {
            val newFabVisible = newDay <= LocalDate.now()
            val bleedingEntryOfDay = bleedingRepository.getBleedingEntryOfDay(newDay)
            val sexEntries = sexRepository.getSexWithContraceptivesOfDate(newDay)
            val pillEntry = pillRepository.getPillOfDay(newDay)

            val pillItemFlow = pillEntry.map { pill ->
                if (pill != null) {
                    return@map listOf(
                        TrackedItem(
                            id = pill.id!!,
                            loggedAt = LocalDateTime.parse(
                                pill.loggedAt, DateTimeFormatter.ISO_DATE_TIME
                            ) ?: LocalDateTime.now(),
                            values = mapOf("type" to listOf("Birth control")),
                            cardEntryItem = CardEntryItem.Medication
                        )
                    )
                }
                listOf()
            }
            val bleedingItemFlow = bleedingEntryOfDay.map { bleedingEntry ->
                if (bleedingEntry != null) {
                    val bleedingStrength =
                        if (bleedingEntry.strength != null) bleedingEntry.strength.toString() else "Not logged"
                    return@map listOf(
                        TrackedItem(
                            id = bleedingEntry.id!!,
                            loggedAt = LocalDateTime.parse(
                                bleedingEntry.loggedAt, DateTimeFormatter.ISO_DATE_TIME
                            ) ?: LocalDateTime.now(), values = mapOf(
                                "strength" to listOf(bleedingStrength)
                            ), cardEntryItem = CardEntryItem.Bleeding
                        )
                    )
                }
                listOf()
            }
            val sexItemsFlow = sexEntries.map { entries ->
                entries.map { entry ->
                    TrackedItem(
                        id = entry.sex.id!!,
                        loggedAt = LocalDateTime.parse(
                            entry.sex.loggedAt, DateTimeFormatter.ISO_DATE_TIME
                        ) ?: LocalDateTime.now(),
                        values = mapOf("type" to listOf(entry.sex.type.toString()),
                            "contraceptives" to entry.contraceptives.map {
                                it.contraceptive.toString()
                            }),
                        cardEntryItem = CardEntryItem.Sex
                    )
                }
            }

            val bleedingAndSex = bleedingItemFlow.combine(sexItemsFlow, transform = { f1, f2 ->
                val listCombined = f1 + f2
                listCombined.sortedBy { it.loggedAt }
            })

            val dayEntries = bleedingAndSex.combine(pillItemFlow, transform = { f1, f2 ->
                val listCombined = f1 + f2
                listCombined.sortedBy { it.loggedAt }
            })


            _uiState.update { currentState ->
                currentState.copy(
                    selectedDay = newDay, dayEntries = dayEntries, fabVisible = newFabVisible
                )
            }
        }
    }

    fun onDismissed(value: TrackedItem) {
        print(value.id)
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                when (value.cardEntryItem.type) {
                    TrackingTypes.BLEEDING -> bleedingRepository.deleteBleedingById(value.id)
                    TrackingTypes.SEX -> sexRepository.deleteSexById(value.id)
                    TrackingTypes.MOOD -> TODO()
                    TrackingTypes.MEDICATION -> pillRepository.deletePillById(value.id)
                }
            }
        }

    }

    fun onMonthChanged(newMonth: YearMonth) {
        fetchBleeding?.cancel()
        fetchBleeding = viewModelScope.launch {
            val newBleedingDays = withContext(Dispatchers.IO) {
                getAllBleedingDaysOfMonth(newMonth)
            }
            val newPredictedBleedingDays =
                menstruationCycleTracker.predictPeriodDaysOfMonth(newMonth)
            _uiState.update { currentState ->
                currentState.copy(
                    currentMonth = newMonth,
                    bleedingDays = newBleedingDays,
                    bleedingDaysPredicted = newPredictedBleedingDays
                )
            }
        }
    }
}