package com.ngrob.android.bluemoon.features.dashboard.screens

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.ngrob.android.bluemoon.MenstruationCycleTracker
import com.ngrob.android.bluemoon.core.data.repository.BleedingRepository
import com.ngrob.android.bluemoon.core.data.repository.PillRepository
import com.ngrob.android.bluemoon.core.data.repository.UserDataRepository
import com.ngrob.android.bluemoon.core.database.model.Pill
import com.ngrob.android.bluemoon.core.model.PillRecipe
import com.ngrob.android.bluemoon.core.model.UserData
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.Job
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.emptyFlow
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.flowOf
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.time.LocalDate
import java.time.temporal.ChronoUnit
import java.util.stream.Collectors
import java.util.stream.LongStream
import javax.inject.Inject


enum class MainButtonState(private val displayText: String) {
    PERIOD_STARTED("Period started"),
    PERIOD_ENDED("Period ended"),
    EDIT_PERIOD("Edit period");

    override fun toString(): String {
        return displayText
    }
}

data class DashboardUiState(
    val bleedingDays: Flow<List<LocalDate>> = emptyFlow(),
    val cycleTracker: MenstruationCycleTracker = MenstruationCycleTracker(listOf()),
    val mainButtonState: MainButtonState = MainButtonState.PERIOD_STARTED,
    val showPillAction: Boolean = false,
    val pillState: PillState? = null
)

data class PillState(
    val todayCount: Int,
    val pillTaken: Boolean,
    val pillRecipe: PillRecipe,
    val isIntakePaused: Boolean
)

@OptIn(ExperimentalCoroutinesApi::class)
@HiltViewModel
class DashboardViewModel @Inject constructor(
    private val bleedingRepository: BleedingRepository,
    private val pillRepository: PillRepository,
    private val userDataRepository: UserDataRepository,
) : ViewModel() {
    private val _uiState = MutableStateFlow(DashboardUiState())
    val uiState = _uiState.asStateFlow()

    private var fetchBleeding: Job? = null

    init {
        getAllBleedingDates()
        viewModelScope.launch {
            _uiState.collect {
                it.bleedingDays.collect { bleedingDates ->
                    val newCycleTracker = MenstruationCycleTracker(bleedingDates)
                    // Init Button state
                    val recentCycle = newCycleTracker.getRecentCycle()
                    val meanCycleLength = newCycleTracker.getPeriodLengthMean()
                    val newButtonState =
                        if (recentCycle == null || recentCycle.endDate.plusDays(meanCycleLength.toLong()) <= LocalDate.now()) {
                            MainButtonState.PERIOD_STARTED
                        } else if (recentCycle.endDate <= LocalDate.now()) {
                            MainButtonState.EDIT_PERIOD
                        } else {
                            MainButtonState.PERIOD_ENDED
                        }
                    _uiState.update { currentState ->
                        currentState.copy(
                            cycleTracker = newCycleTracker,
                            mainButtonState = newButtonState
                        )
                    }
                }
            }
        }
        viewModelScope.launch {
            userDataRepository.userData.flatMapLatest { userData ->
                if (userData.trackBirthControl) {
                    pillRepository.getPillOfDay(LocalDate.now()).map { pill ->
                        // Process your data here and return a new state
                        setPillState(userData, pill)
                    }
                } else {
                    flowOf(
                        DashboardUiState(showPillAction = false)
                    )
                }
            }.collect { uiState ->
                _uiState.update {
                    it.copy(
                        showPillAction = uiState.showPillAction,
                        pillState = uiState.pillState
                    )

                }
            }
        }

    }


    private fun getAllBleedingDates() {
        fetchBleeding?.cancel()
        fetchBleeding = viewModelScope.launch {
            val newBleedingDays = withContext(Dispatchers.IO) {
                bleedingRepository.getAllBleedingDates()
            }

            _uiState.update { currentState ->
                currentState.copy(
                    bleedingDays = newBleedingDays,
                )
            }
        }
    }

    private fun setPillState(userData: UserData, pill: Pill?): DashboardUiState {
        val pillRecipe = userData.pillRecipe
        val isIntakePaused =
            (ChronoUnit.DAYS.between(pillRecipe.firstIntake, LocalDate.now())
                .toInt() + 1) % 29 > pillRecipe.numberOfPills
        val todayCount =
            (ChronoUnit.DAYS.between(pillRecipe.firstIntake, LocalDate.now())
                .toInt() + 1) % 29 % (pillRecipe.numberOfPills + 1)
        val pillTaken = pill != null


        return DashboardUiState(
            showPillAction = true,
            pillState = PillState(
                todayCount = todayCount,
                pillRecipe = pillRecipe,
                isIntakePaused = isIntakePaused,
                pillTaken = pillTaken
            )
        )

    }

    fun startPeriod() {
        val meanPeriodLength = _uiState.value.cycleTracker.getPeriodLengthMean()

        val bleedingDatesToAdd = LongStream.iterate(0) { i -> i + 1 }
            .limit(meanPeriodLength.toLong())
            .mapToObj { i ->
                LocalDate.now().plusDays(
                    i
                )
            }
            .collect(Collectors.toList())

        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                bleedingRepository.addBleedings(bleedingDatesToAdd)
            }
        }

        _uiState.update { currentState ->
            currentState.copy(
                mainButtonState = MainButtonState.PERIOD_ENDED
            )
        }

    }

    fun endPeriod() {
        //TODO Maybe delete bleeding dates which are in future but where automatically created upon start period button press
        _uiState.update { currentState ->
            currentState.copy(
                mainButtonState = MainButtonState.EDIT_PERIOD
            )
        }
    }

    fun takePill() {
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                pillRepository.takePill(LocalDate.now(), isSubstitute = false)
            }
        }
    }
}
